# -*- coding: utf-8 -*-

import sysactions
from bustoken import TK_POS_UPDATER_PERFORM_USER_UPDATE
from msgbus import TK_SYS_NAK, FM_PARAM
from actions.util.list_users import list_ids_opened_users
from utilfunctions import json_serialize


@sysactions.action
def update_users(pos_id):
    data = json_serialize(list_ids_opened_users())
    msg = sysactions.send_message("PosUpdater", TK_POS_UPDATER_PERFORM_USER_UPDATE, FM_PARAM, data)
    if msg.token == TK_SYS_NAK:
        message = "$UPDATE_USERS_" + msg.data.upper()
        sysactions.show_messagebox(pos_id, message, buttons="OK")
        return

    sysactions.show_messagebox(pos_id, "$UPDATE_USERS_MESSAGE", buttons="OK")



from commons.report import Generator
from reports_app.cashtransfersummary.dto import CashTransferSummaryInfo  # noqa
from reports_app.cashtransfersummary.dto import CashTransferSummaryDto
from reports_app.cashtransfersummary.dto import CashTransferSummaryHeaderDto
from reports_app.cashtransfersummary.dto import CashTransferSummaryBodyDto


class CashTransferGenerator(Generator):
    def __init__(self, cash_transfer_summary_info):
        # type: (CashTransferSummaryInfo) -> ()
        self.cash_transfer_summary_info = cash_transfer_summary_info

    def generate_data(self):
        # type: (CashTransferSummaryInfo) -> CashTransferSummaryDto
        cash_transfer_summary_header_dto = CashTransferSummaryHeaderDto(
            self.cash_transfer_summary_info.cash_out,
            self.cash_transfer_summary_info.pos_id,
            self.cash_transfer_summary_info.store_id,
            self.cash_transfer_summary_info.operator_id,
            self.cash_transfer_summary_info.business_date
        )
        cash_transfer_summary_body_dto = CashTransferSummaryBodyDto(
            self.cash_transfer_summary_info.cash_out,
            self.cash_transfer_summary_info.value,
            self.cash_transfer_summary_info.envelope_number,
            self.cash_transfer_summary_info.authorizer
        )

        return CashTransferSummaryDto(cash_transfer_summary_header_dto, cash_transfer_summary_body_dto)



# -*- coding: utf-8 -*-

from xml.etree import cElementTree as eTree
from datetime import datetime

import bustoken
import sysactions
from msgbus import TK_SYS_ACK, FM_PARAM
from mw_helper import show_message_dialog, show_numpad_dialog

from .get_operator_level import get_user_level
from ..models import UserLevels


def get_authorization(pos_id,
                      min_level=None,
                      model=None,
                      timeout=60000,
                      can_bypass_reader=False,
                      insert_auth=False,
                      order_id=None,
                      display_title="$USER_AUTHENTICATION_WITH_PARAMETER",
                      user_id=None,
                      executed_function=None):

    # Fingerprint
    try:
        ret = sysactions.mbcontext.MB_EasySendMessage("FingerPrintReader{0}".format(pos_id), bustoken.TK_FPR_OK)
        if ret.token == TK_SYS_ACK:
            ret = sysactions.mbcontext.MB_EasySendMessage(
                "FingerPrintReader{0}".format(pos_id),
                bustoken.TK_FPR_AUTHORIZE_USER,
                format=FM_PARAM,
                data=pos_id)

            if ret.token == bustoken.TK_FPR_TIMEOUT and can_bypass_reader and not user_id:
                return get_authorization_original(pos_id, UserLevels.MANAGER.value, model, timeout, display_title)
            if ret.token != TK_SYS_ACK:
                sysactions.show_messagebox(pos_id, "$ERROR_READING_FINGERPRINT", "$ERROR")
            elif ret.data == "":
                sysactions.show_messagebox(pos_id, "$NOT_VERIFIED_USER_FINGERPRINT", "$ERROR")
            else:
                user_xml_str = None
                user_id = user_id or ret.data
                try:
                    user_xml_str = sysactions.get_user_information(user_id)
                except Exception as _:
                    sysactions.show_messagebox(pos_id, "$ASSOCIATED_FINGERPRINT_ERROR", "$ERROR")

                if user_xml_str is None:
                    sysactions.show_messagebox(pos_id, "$ASSOCIATED_FINGERPRINT_ERROR", "$ERROR")

                user_xml = eTree.XML(user_xml_str)
                user_element = user_xml.find("user")
                user_level = int(user_element.attrib["Level"])
                if user_level < min_level:
                    sysactions.show_messagebox(pos_id, "$ACCESS_DENIED", "$INFORMATION")
    except Exception as _:
        pass

    # Se chegamos aqui, nao temos leitor de impressao digital ativo, fallback para a forma tradicional
    response = get_authorization_original(pos_id, min_level, model, timeout, display_title, user_id)
    if response[0] and (insert_auth or executed_function is not None):
        if not model:
            model = sysactions.get_model(pos_id)
        pos_ot = sysactions.get_posot(model)
        user_id = sysactions.get_custom(model, 'Last Manager ID')
        if not order_id:
            order = model.find("CurrentOrder/Order")
            order_id = order.get("orderId")

        if order_id:
            if insert_auth:
                pos_ot.setOrderCustomProperty("AUTHENTICATION_USER", user_id, order_id)

            if executed_function is not None:
                property_name = "{}: {}".format(executed_function, datetime.now().strftime("%m/%d/%Y %H:%M:%S"))
                pos_ot.setOrderCustomProperty(property_name, user_id, order_id)

    return response


def get_authorization_original(pos_id, min_level=None, model=None, timeout=60000, display_title="", user_id=None):
    auth_type = sysactions.get_cfg(pos_id).key_value("authorizationType", "card").lower()

    if auth_type == "card":
        model = model or sysactions.get_model(pos_id)
        msr_name = sysactions.get_used_service(model, "msr")
        if not msr_name:
            show_message_dialog(pos_id, message="$CARDREADER_NOT_CONFIGURED")
            return False, None

        dlgid = sysactions.show_any_dialog(pos_id, "messageDialog", "$SWIPE_CARD_TO_AUTHORIZE",
                                           "$MAGNETIC_STRIPE_READER", "", "$CANCEL", "swipecard", timeout, "", "", "",
                                           "", None, True)

        try:
            # Read tracks from the card reader
            tracks = sysactions.read_msr(msr_name, timeout, dlgid)
        finally:
            sysactions.close_asynch_dialog(pos_id, dlgid)

        if tracks:
            # TODO: parse tracks and validate level if necessary
            return True, None

        return False, None

    elif auth_type == "login":
        if display_title == "$USER_AUTHENTICATION_WITH_PARAMETER":
            model = model or sysactions.get_model(pos_id)
            user_level_name = UserLevels.get_name(min_level)
            if user_level_name is None:
                user_levels = UserLevels.list_values()
                for level in user_levels:
                    if min_level <= level:
                        user_level_name = UserLevels.get_name(level)
                        break

            display_title = "$USER_AUTHENTICATION"
            if user_level_name is not None:
                level_name = sysactions.translate_message(model, user_level_name)
                display_title = "$USER_AUTHENTICATION_WITH_PARAMETER|{}".format(level_name)

        if not user_id:
            user_id = show_numpad_dialog(pos_id, display_title, "$ENTER_USER_ID")
            if user_id is None:
                return False, None
            if user_id == '':
                show_message_dialog(pos_id, message="$INVALID_USER")
                return False, None

        password = show_numpad_dialog(pos_id, display_title, "$ENTER_PASSWORD|{}".format(user_id), "password")
        if password is None:
            return False, None

        try:
            sysactions.authenticate_user(user_id, password, min_level=min_level)
            sysactions.set_custom(pos_id, 'Last Manager ID', user_id)
            sysactions.set_custom(pos_id, 'Authorization Level', get_user_level(user_id))
            sysactions.sys_log_info("Authentication succeeded for POS id: {}, user id: {}".format(pos_id, user_id))
            return True, user_id
        except sysactions.AuthenticationFailed:
            show_message_dialog(pos_id, message="$INVALID_USERNAME_OR_PASSWORD")

        return False, None

    return False, None



from pos_model import Order, SaleItem
from typing import Dict, Any

from _ContextKeys import ContextKeys
from _NfceXmlPartBuilder import NfceXmlPartBuilder


class DetTipBuilder(NfceXmlPartBuilder):

    def __init__(self, det_builder, tip_config):
        # type: (NfceXmlPartBuilder, dict) -> None
        self.det_builder = det_builder
        self.tip_config = tip_config

    def build_xml(self, order, context):
        # type: (Order, Dict[unicode, Any]) -> unicode
        xml = ""

        if order.tip and self.tip_config['TipCode'] and self.tip_config['TipName']:
            sale_item = SaleItem()
            sale_item.part_code = self.tip_config['TipCode']
            sale_item.product_name = self.tip_config['TipName'].upper()
            sale_item.quantity = 1
            sale_item.unit_price = order.tip
            sale_item.item_price = order.tip

            context[ContextKeys.current_sale_item] = sale_item
            xml += self.det_builder.build_xml(order, context)

        return xml



from ._CustomException import \
    ConfigurationReaderException, \
    NotValidBenefit, \
    ErrorLockingCoupon, \
    ErrorUnlockingCoupon, \
    ErrorBurningCoupon, \
    ErrorObtainingCouponInfo, \
    ErrorApplyingBenefit, \
    BenefitApplierNotFound, \
    BenefitIdNotFound



class ConfigurationReaderException(Exception):
    pass


class NotValidBenefit(Exception):
    pass


class ErrorLockingCoupon(Exception):
    pass


class ErrorUnlockingCoupon(Exception):
    pass


class ErrorBurningCoupon(Exception):
    pass


class ErrorObtainingCouponInfo(Exception):
    pass


class ErrorApplyingBenefit(Exception):
    pass


class BenefitApplierNotFound(Exception):
    pass


class BenefitIdNotFound(Exception):
    pass


class ErrorCheckingVoucher(Exception):
    pass



from ._Message import Message
from ._AckMessage import AckMessage
from ._NakMessage import NakMessage
from ._SwaggerMessage import SwaggerMessage
from ._JsonSwaggerMessage import JsonSwaggerMessage
from ._Event import Event
from ._MessageBus import MessageBus
from ._TokenCreator import TokenCreator
from ._TokenPriority import TokenPriority
from ._DataType import DataType
from ._DefaultToken import DefaultToken
from ._MessageBusExceptionType import MessageBusExceptionType
from ._MessageBusException import MessageBusException


__all__ = ["Message",
           "AckMessage",
           "NakMessage",
           "SwaggerMessage",
           "JsonSwaggerMessage",
           "MessageBus",
           "Event",
           "TokenCreator",
           "TokenPriority",
           "DataType",
           "DefaultToken",
           "MessageBusExceptionType",
           "MessageBusException"]



from typing import Any

from messageprocessorutil import UuidMessageProcessor

from nperipherals_core.interactor import HealthCheckInteractor


class HealthCheckProcessor(UuidMessageProcessor):
    def __init__(self, interactor):
        # type: (HealthCheckInteractor) -> None

        super(HealthCheckProcessor, self).__init__()
        self.interactor = interactor

    def parse_data(self, data, event_type):
        # type: (Any, Any) -> Any

        return

    def call_business(self, obj):
        # type: (Any) -> None
        return self.interactor.restart_dead_threads()

    def format_response(self, message_bus, message, event, input_obj, result):
        return

    def format_exception(self, message_bus, message, event, input_obj, exception):
        return

    def format_parse_exception(self, message_bus, message, event, data, exception):
        return




class StaticConfig(object):
    def __init__(self,
                 time_to_alert_table_opened,
                 time_to_alert_table_is_idle,
                 time_to_alert_table_is_idle_warning,
                 time_to_alert_recall_delivery_is_idle,
                 recall_button,
                 operator_button,
                 special_menus,
                 enabled_tags,
                 screen_timeout,
                 cancel_timeout_window,
                 show_in_dashboard,
                 products_screen_dimensions,
                 spinner_config,
                 enable_tab_btns,
                 enable_pre_start_sale,
                 available_sale_types,
                 fetch_stored_orders_timeout,
                 tender_types,
                 price_override_enabled,
                 can_open_table_from_another_operator,
                 can_edit_order,
                 sat_info,
                 navigation_config,
                 show_cash_in_and_cash_out,
                 show_void_closed_order,
                 min_level_needed_to_see_all_tables,
                 totem_config,
                 cash_payment_enabled,
                 coupons_enabled,
                 discounts_enabled,
                 bill_payment_enabled,
                 special_modifiers,
                 show_ruptured_products,
                 pos_navigation,
                 remote_order_status,
                 delivery_sound,
                 delivery_address,
                 chat_enabled,
                 services_period,
                 menu_test,
                 pre_account_with_discount,
                 main_navigation):
        self.time_to_alert_table_opened = time_to_alert_table_opened
        self.time_to_alert_table_is_idle = time_to_alert_table_is_idle
        self.time_to_alert_table_is_idle_warning = time_to_alert_table_is_idle_warning
        self.time_to_alert_recall_delivery_is_idle = time_to_alert_recall_delivery_is_idle
        self.recall_button = recall_button
        self.operator_button = operator_button
        self.special_menus = special_menus
        self.enabled_tags = enabled_tags
        self.screen_timeout = screen_timeout
        self.cancel_timeout_window = cancel_timeout_window
        self.show_in_dashboard = show_in_dashboard
        self.products_screen_dimensions = products_screen_dimensions
        self.spinner_config = spinner_config
        self.enable_tab_btns = enable_tab_btns
        self.enable_pre_start_sale = enable_pre_start_sale
        self.available_sale_types = available_sale_types
        self.fetch_stored_orders_timeout = fetch_stored_orders_timeout
        self.tender_types = tender_types
        self.price_override_enabled = price_override_enabled
        self.can_open_table_from_another_operator = can_open_table_from_another_operator
        self.can_edit_order = can_edit_order
        self.sat_info = sat_info
        self.navigation_config = navigation_config
        self.show_cash_in_and_cash_out = show_cash_in_and_cash_out
        self.show_void_closed_order = show_void_closed_order
        self.min_level_needed_to_see_all_tables = min_level_needed_to_see_all_tables
        self.totem_config = totem_config
        self.cash_payment_enabled = cash_payment_enabled
        self.coupons_enabled = coupons_enabled
        self.discounts_enabled = discounts_enabled
        self.bill_payment_enabled = bill_payment_enabled
        self.special_modifiers = special_modifiers
        self.show_ruptured_products = show_ruptured_products
        self.pos_navigation = pos_navigation
        self.remote_order_status = remote_order_status
        self.delivery_sound = delivery_sound
        self.delivery_address = delivery_address
        self.chat_enabled = chat_enabled
        self.services_period = services_period
        self.menu_test = menu_test
        self.pre_account_with_discount = pre_account_with_discount
        self.main_navigation = main_navigation



from ..http import dump_header
from ..http import parse_set_header
from ..utils import environ_property
from ..utils import header_property


class CORSRequestMixin(object):
    """A mixin for :class:`~werkzeug.wrappers.BaseRequest` subclasses
    that adds descriptors for Cross Origin Resource Sharing (CORS)
    headers.

    .. versionadded:: 1.0
    """

    origin = environ_property(
        "HTTP_ORIGIN",
        doc=(
            "The host that the request originated from. Set"
            " :attr:`~CORSResponseMixin.access_control_allow_origin` on"
            " the response to indicate which origins are allowed."
        ),
    )

    access_control_request_headers = environ_property(
        "HTTP_ACCESS_CONTROL_REQUEST_HEADERS",
        load_func=parse_set_header,
        doc=(
            "Sent with a preflight request to indicate which headers"
            " will be sent with the cross origin request. Set"
            " :attr:`~CORSResponseMixin.access_control_allow_headers`"
            " on the response to indicate which headers are allowed."
        ),
    )

    access_control_request_method = environ_property(
        "HTTP_ACCESS_CONTROL_REQUEST_METHOD",
        doc=(
            "Sent with a preflight request to indicate which method"
            " will be used for the cross origin request. Set"
            " :attr:`~CORSResponseMixin.access_control_allow_methods`"
            " on the response to indicate which methods are allowed."
        ),
    )


class CORSResponseMixin(object):
    """A mixin for :class:`~werkzeug.wrappers.BaseResponse` subclasses
    that adds descriptors for Cross Origin Resource Sharing (CORS)
    headers.

    .. versionadded:: 1.0
    """

    @property
    def access_control_allow_credentials(self):
        """Whether credentials can be shared by the browser to
        JavaScript code. As part of the preflight request it indicates
        whether credentials can be used on the cross origin request.
        """
        return "Access-Control-Allow-Credentials" in self.headers

    @access_control_allow_credentials.setter
    def access_control_allow_credentials(self, value):
        if value is True:
            self.headers["Access-Control-Allow-Credentials"] = "true"
        else:
            self.headers.pop("Access-Control-Allow-Credentials", None)

    access_control_allow_headers = header_property(
        "Access-Control-Allow-Headers",
        load_func=parse_set_header,
        dump_func=dump_header,
        doc="Which headers can be sent with the cross origin request.",
    )

    access_control_allow_methods = header_property(
        "Access-Control-Allow-Methods",
        load_func=parse_set_header,
        dump_func=dump_header,
        doc="Which methods can be used for the cross origin request.",
    )

    access_control_allow_origin = header_property(
        "Access-Control-Allow-Origin",
        doc="The origin or '*' for any origin that may make cross origin requests.",
    )

    access_control_expose_headers = header_property(
        "Access-Control-Expose-Headers",
        load_func=parse_set_header,
        dump_func=dump_header,
        doc="Which headers can be shared by the browser to JavaScript code.",
    )

    access_control_max_age = header_property(
        "Access-Control-Max-Age",
        load_func=int,
        dump_func=str,
        doc="The maximum age in seconds the access control settings can be cached for.",
    )



# encoding: utf-8
from commons.dto import AlignTypes, ReportColumnDefinition
from commons.report import Formatter
from commons.util import TableReport, Translator
from commons.util import EscPosQrCodeGenerator  # noqa
from reports_app.picklist.dto import PickListDto, PickListItem  # noqa

ESC = b'\x1b'
TXT_BOLD_OFF = ESC + b'\x45\x00'  # Bold font OFF
TXT_BOLD_ON = ESC + b'\x45\x01'  # Bold font ON
TXT_ALIGN_CT = ESC + b'\x61\x01'  # Centering
TXT_ALIGN_LT = ESC + b'\x61\x00'  # Left justification


class PickListFormatter(Formatter):

    def __init__(self, formatter_type, qrcode_string_before, qrcode_string_after, show_qrcode):
        super(PickListFormatter, self).__init__()
        self.formatter_type = formatter_type
        self.qrcode_string_before = qrcode_string_before
        self.qrcode_string_after = qrcode_string_after
        self.show_qrcode = show_qrcode
        self.translated_messages = {}

    def format_report(self, pick_list_dto, translator):
        # type: (PickListDto, Translator) -> str

        self.translated_messages = translator.translate_labels(
                [
                    "$STORE", "$DATE_HOUR", "$ORDER_NUMBER", "$CUSTOMER_NAME", "$EXTRA", "$WITHOUT", "$DRIVE_THRU",
                    "$TAKE_OUT", "$TOTEM", "$DELIVERY", "$ADDRESS_CAP", "$REFERENCE", "$NEIGHBORHOOD", "$ZIPCODE",
                    "$CITY", "$STATE", "$PARTNER", "$PARTNER_NUMBER"
                ], pick_list_dto.pos_id
        )

        report = self._validate_and_create_header_and_body(pick_list_dto)
        report_bytes = report.encode("utf-8")
        report_bytes = self._append_qrcode(pick_list_dto, report_bytes)
        return report_bytes

    def _validate_and_create_header_and_body(self, pick_list_dto):
        self._validate_header_body(pick_list_dto)
        report = self._append_header(pick_list_dto)
        report = self._choose_type_body_format(pick_list_dto, report)
        return report

    def _append_qrcode(self, pick_list_dto, report_bytes):
        if self.show_qrcode:
            report_bytes += ("%(TXT_ALIGN_CT)s" % self._join(globals(), locals()))
            report_bytes += EscPosQrCodeGenerator.generate_qr_code(self.qrcode_string_before +
                                                                   str(pick_list_dto.pick_list_header.order_id).zfill(8) +
                                                                   self.qrcode_string_after)
            report_bytes += ("%(TXT_ALIGN_LT)s" % self._join(globals(), locals()))
            report_bytes = self._add_final_line(report_bytes)

        return report_bytes

    @staticmethod
    def _add_final_line(report_bytes):
        report_lines = [[ReportColumnDefinition(fill_with_char=' ')], [ReportColumnDefinition(fill_with_char='=')]]
        table_formatter = TableReport([ReportColumnDefinition(width=38)])
        report_bytes += table_formatter.format(report_lines)
        return report_bytes

    def _choose_type_body_format(self, pick_list_dto, report):
        if str(self.formatter_type) == 'normal':
            report += self._append_body(pick_list_dto)
        else:
            report += self._append_body_grouped(pick_list_dto)
        return report

    @staticmethod
    def _validate_header_body(pick_list_dto):
        if pick_list_dto.pick_list_header is None:
            raise PickListFormatter.InvalidDto(PickListFormatter.InvalidDto.HeaderIsNone)
        if pick_list_dto.pick_list_body is None:
            raise PickListFormatter.InvalidDto(PickListFormatter.InvalidDto.BodyIsNone)

    def _append_header(self, pick_list_dto):
        # type: (PickListDto) -> str
        report_lines = []

        self._append_data_header(pick_list_dto, report_lines)

        table_formatter = TableReport(lines=report_lines,
                                      column_definitions=[
                                          ReportColumnDefinition(width=12, fill_with_char='.'),
                                          ReportColumnDefinition(width=26, before_text=': ')
                                      ]
                                      )

        return table_formatter.generate_parts()

    def _append_data_header(self, pick_list_dto, report_lines):
        self._append_line_separator(report_lines)
        self._append_title(report_lines, pick_list_dto)
        self._append_current_date(report_lines, pick_list_dto.pick_list_header)
        self._append_order_id_and_store(report_lines, pick_list_dto.pick_list_header)
        self._append_client_name(report_lines, pick_list_dto.pick_list_header)
        self._append_data_address(report_lines, pick_list_dto)

    @staticmethod
    def _append_line_separator(report_lines):
        report_lines.append([ReportColumnDefinition(
            fill_with_char='='
        )])

    def _append_title(self, report_lines, pick_list_dto):
        title = self._get_title(pick_list_dto)
        report_lines.append([ReportColumnDefinition(
            text="(" + self.translated_messages[title].upper() + ")",
            align=AlignTypes.CENTER
        )])

    @staticmethod
    def _get_title(pick_list_dto):
        title = "$STORE"
        if pick_list_dto.pod_type == "DT":
            title = "$DRIVE_THRU".upper()
        if pick_list_dto.pod_type == "TT":
            title = "$TOTEM"
        if pick_list_dto.pod_type == "DL":
            title = "$DELIVERY"
        return title

    def _append_data_address(self, report_lines, pick_list_dto):

        if pick_list_dto.pod_type == "DL":

            list_keys_to_sort = [self.translated_messages["$ADDRESS_CAP"],
                                 self.translated_messages["$REFERENCE"],
                                 self.translated_messages["$NEIGHBORHOOD"],
                                 self.translated_messages["$ZIPCODE"],
                                 self.translated_messages["$CITY"],
                                 self.translated_messages["$STATE"],
                                 self.translated_messages["$PARTNER"],
                                 self.translated_messages["$PARTNER_NUMBER"]]

            dict_address_values = {self.translated_messages["$ADDRESS_CAP"]: "##ADDRESS##",
                                   self.translated_messages["$REFERENCE"]: "##referencia com mais de trinta e oito"
                                                                           " caracteres para teste e com mais "
                                                                           "de duas linhas##",
                                   self.translated_messages["$NEIGHBORHOOD"]: "##NEIGHBORHOOD##",
                                   self.translated_messages["$ZIPCODE"]: "##ZIPCODE##",
                                   self.translated_messages["$CITY"]: "##CITY##",
                                   self.translated_messages["$STATE"]: "##STATE##",
                                   self.translated_messages["$PARTNER"]: "##PARTNER##",
                                   self.translated_messages["$PARTNER_NUMBER"]: "##PARTNER_NUMBER##"}
            for key in list_keys_to_sort:
                self._append_lines_address(dict_address_values, key, report_lines)

    def _append_lines_address(self, dict_address_values, key, report_lines):
        size_text = 25
        parts = len(dict_address_values.get(key)) / float(size_text)
        report_lines.append([key, dict_address_values.get(key)[:size_text]])
        if parts > 1:
            self._line_break_address(dict_address_values, key, parts, report_lines, size_text)

    @staticmethod
    def _line_break_address(dict_address_values, key, parts, report_lines, size_text):
        for part in range(int(parts) - 1):
            text = dict_address_values.get(key)[25 + (size_text * part):(size_text * (part + 1)) + 38]
            report_lines.append([ReportColumnDefinition(text=text, align=AlignTypes.LEFT, width=38)])
            size_text = 38

    def _append_current_date(self, report_lines, pick_list_header_dto):
        current_date = pick_list_header_dto.date.strftime("%d/%m/%Y %H:%M:%S")
        report_lines.append([self.translated_messages["$DATE_HOUR"], current_date])

    def _append_order_id_and_store(self, report_lines, pick_list_header_dto):
        order_number = str(pick_list_header_dto.order_id) + " / " + str(pick_list_header_dto.store)
        report_lines.append([self.translated_messages["$ORDER_NUMBER"], order_number])

    def _append_client_name(self, report_lines, pick_list_header_dto):
        if pick_list_header_dto.client_name is None:
            pick_list_header_dto.client_name = ""

        report_lines.append([self.translated_messages["$CUSTOMER_NAME"], pick_list_header_dto.client_name])

    def _append_body(self, pick_list_dto):
        # type: (PickListDto) -> str
        report_lines = []
        self._append_data_body(pick_list_dto, report_lines)
        table_formatter = TableReport([ReportColumnDefinition(width=38, fill_with_char=' ')])
        return table_formatter.format(report_lines)

    def _append_body_grouped(self, pick_list_dto):
        # type: (PickListDto) -> str
        report_lines = []
        self._append_init_body(report_lines, pick_list_dto)
        self._create_data_to_format_grouped(pick_list_dto, report_lines)
        self._append_end_body(report_lines, pick_list_dto)

        table_formatter = TableReport([ReportColumnDefinition(width=38, fill_with_char=' ')])
        return table_formatter.format(report_lines)

    def _append_data_body(self, pick_list_dto, report_lines):
        self._append_init_body(report_lines, pick_list_dto)
        for pick_list_item in pick_list_dto.pick_list_body.pick_list_items:
            text_space = ""
            self._append_item_info(report_lines, pick_list_item, text_space, True, None, False)
        self._append_end_body(report_lines, pick_list_dto)

    def _append_init_body(self, report_lines, pick_list_dto):
        self._append_line_separator(report_lines)
        if pick_list_dto.pod_type == "DT" or pick_list_dto.pod_type == "DL":
            self._append_pod_type(report_lines, pick_list_dto.pod_type)
        elif pick_list_dto.sale_type != "EAT_IN":
            self._append_sale_type(report_lines, pick_list_dto.sale_type)
        self._append_empty_line(report_lines)

    def _append_end_body(self, report_lines, pick_list_dto):
        self._append_empty_line(report_lines)
        if pick_list_dto.pod_type == "DT" or pick_list_dto.pod_type == "DL":
            self._append_pod_type(report_lines, pick_list_dto.pod_type)
        elif pick_list_dto.sale_type != "EAT_IN":
            self._append_sale_type(report_lines, pick_list_dto.sale_type)
        self._append_empty_line(report_lines)

    def _append_pod_type(self, report_lines, pod_type):
        if pod_type == "DT":
            report_lines.append([ReportColumnDefinition(
                text=self.translated_messages["$DRIVE_THRU"],
                align=AlignTypes.CENTER,
                fill_with_char="*"
            )])
        if pod_type == "DL":
            report_lines.append([ReportColumnDefinition(
                text=" " + self.translated_messages["$DELIVERY"].upper() + " ",
                align=AlignTypes.CENTER,
                fill_with_char="*"
            )])

    def _append_sale_type(self, report_lines, sale_type):
        if sale_type == "TAKE_OUT":
            report_lines.append([ReportColumnDefinition(
                text=" " + self.translated_messages["$TAKE_OUT"] + " ",
                align=AlignTypes.CENTER,
                fill_with_char="*"
            )])

    def _create_data_to_format_grouped(self, pick_list_dto, report_lines):
        list_items = []  # type: List[PickListItem]
        list_items_grouped = []
        self._create_list_items(list_items, pick_list_dto.pick_list_body)
        self._create_list_items_grouped(list_items, list_items_grouped)
        self._create_lines_grouped(list_items, list_items_grouped, report_lines)

    def _create_lines_grouped(self, list_items, list_items_grouped, report_lines):
        for j in list_items_grouped:
            qty_item = 0
            self._create_lines_items(j, list_items, qty_item, report_lines)
            self._create_lines_sub_items(j, report_lines)

    def _create_lines_sub_items(self, j, report_lines):
        for sub_item in j.sub_items:
            if int(sub_item.qty) == 0:
                line = self.translated_messages["$WITHOUT"] + " " + sub_item.description
            else:
                line = "{} {} {}".format(str(int(sub_item.qty) - int(sub_item.default_qty)),
                                         self.translated_messages["$EXTRA"],
                                         sub_item.description)
            text = "%(TXT_BOLD_ON)s%(line)s%(TXT_BOLD_OFF)s" % self._join(globals(), locals())
            report_lines.append([ReportColumnDefinition(text=text, before_text="   ")])

    def _create_lines_items(self, j, list_items, qty_item, report_lines):
        for i in list_items:
            if self._compare_item(j, i):
                if int(i.qty) != 0:
                    qty_item = qty_item + int(j.multiplied_qty)
        self._add_text_item(j, qty_item, report_lines)

    def _add_text_item(self, j, qty_item, report_lines):
        if int(j.qty) != 0:
            line = str(qty_item) + " " + j.description
            text = "%(TXT_BOLD_ON)s%(line)s%(TXT_BOLD_OFF)s" % self._join(globals(), locals())
            report_lines.append([ReportColumnDefinition(text=text)])

    def _create_list_items_grouped(self, list_items, list_items_grouped):
        for i in list_items:
            list_items_grouped.append(i)
        for i in range(0, len(list_items)):
            for j in range(i + 1, len(list_items)):
                self._compare_item_and_remove_equal(i, j, list_items, list_items_grouped)

    def _compare_item_and_remove_equal(self, i, j, list_items, list_items_grouped):
        if self._compare_item(list_items[i], list_items[j]):
            if list_items[j] in list_items_grouped:
                if int(list_items[j].qty) == 0:
                    list_items_grouped.remove(list_items[j])
                else:
                    list_items_grouped.remove(list_items[i])

    def _create_list_items(self, list_items, pick_list_body):
        for pick_list_item in pick_list_body.pick_list_items:
            if pick_list_item.item_type == 'COMBO':
                for sub_item in pick_list_item.sub_items:
                    self._append_sub_items(list_items, sub_item)
            else:
                list_items.append(pick_list_item)

    def _append_sub_items(self, list_items, sub_item):
        if sub_item.item_type == 'OPTION':
            for sub_item_option in sub_item.sub_items:
                list_items.append(sub_item_option)
        else:
            list_items.append(sub_item)

    def _compare_item(self, item1, item2):
        if item1.description == item2.description and len(item1.sub_items) == len(item2.sub_items):
            if len(item1.sub_items) == 0:
                return True
            return self._compare_sub_items(item1.sub_items, item2.sub_items)
        else:
            return False

    def _compare_sub_items(self, sub_items1, sub_items2):
        for i in range(0, len(sub_items1)):
            is_equal = False
            for j in range(0, len(sub_items2)):
                if sub_items1[i].description == sub_items2[j].description and sub_items1[i].qty == sub_items2[j].qty:
                    is_equal = True
                    break
        return is_equal

    def _append_item_info(self, report_lines, pick_list_item, text_space, first, default_qty, is_option_sub_item):
        text = ""
        default_qty, is_option_sub_item, text_space = self._validate_is_option_and_create_text(default_qty, first,
                                                                                               is_option_sub_item,
                                                                                               pick_list_item,
                                                                                               report_lines, text,
                                                                                               text_space)

        for pick_list_item in pick_list_item.sub_items:
            self._append_item_info(report_lines, pick_list_item, text_space, False, default_qty, is_option_sub_item)

    def _validate_is_option_and_create_text(self, default_qty, first, is_option_sub_item, pick_list_item, report_lines,
                                            text, text_space):
        if pick_list_item.item_type == "OPTION":
            default_qty = int(pick_list_item.qty) - int(pick_list_item.default_qty)
            is_option_sub_item = True
        elif (is_option_sub_item and int(pick_list_item.qty) != 0) or not is_option_sub_item:
                default_qty, text_space = self._create_text_and_append_lines(default_qty,
                                                                             first,
                                                                             pick_list_item,
                                                                             report_lines,
                                                                             text,
                                                                             text_space)
        return default_qty, is_option_sub_item, text_space

    def _join(self, dic1, dic2):
        d = dict(dic1)
        d.update(dic2)
        return d

    def _create_text_and_append_lines(self, default_qty, first, pick_list_item, report_lines, text, text_space):
        default_qty, text = self._create_line(default_qty, first, pick_list_item, text)
        report_lines.append([ReportColumnDefinition(text=text, before_text=text_space)])
        text_space += "   "
        return default_qty, text_space

    def _create_line(self, default_qty, first, pick_list_item, text):
        if pick_list_item.item_type == "COMBO" or first:
            line = str(pick_list_item.qty) + " " + pick_list_item.description
            text = "%(TXT_BOLD_ON)s%(line)s%(TXT_BOLD_OFF)s" % self._join(globals(), locals())
        else:
            default_qty, qty_items = self._get_default_qty(default_qty, pick_list_item)
            text = self._add_description(pick_list_item, qty_items, text)
        return default_qty, text

    def _add_description(self, pick_list_item, qty_items, text):
        if qty_items > 0:
            text = self._add_extra_items(pick_list_item, qty_items, text)
        elif qty_items < 0:
            text = self._remove_items(pick_list_item, text)
        else:
            text += pick_list_item.description
        return text

    def _add_extra_items(self, pick_list_item, qty_items, text):
        x = 0
        while x < qty_items:
            text += self.translated_messages["$EXTRA"] + " " + pick_list_item.description
            x = x + 1
        return text

    def _remove_items(self, pick_list_item, text):
        text += self.translated_messages["$WITHOUT"] + " " + pick_list_item.description
        return text

    def _get_default_qty(self, default_qty, pick_list_item):
        if default_qty is None:
            qty_items = int(pick_list_item.qty) - int(pick_list_item.default_qty)
        else:
            qty_items = default_qty
            default_qty = None
        return default_qty, qty_items

    @staticmethod
    def _append_empty_line(report_lines):
        report_lines.append([ReportColumnDefinition(fill_with_char=' ')])

    class InvalidDto(Exception):
        HeaderIsNone = "Header is None"
        BodyIsNone = "Body is None"

        def __init__(self, message):
            super(PickListFormatter.InvalidDto, self).__init__(message)



# -*- coding: utf-8 -*-
def migrate(conn, tables):
    update_tables_priority = ["OperationDescription", "OperationStatus", "PendingOperations"]
    for table in update_tables_priority:
        if tables[table]["fields_OldDB"]:
            str_common_fields = ", ".join(tables[table]["fields_Common"])
            stmt = "INSERT OR REPLACE INTO %s (%s) SELECT %s FROM old.%s;" % (table, str_common_fields, str_common_fields, table)
            conn.query(stmt)



from mw import RealClock, MwTipRepository, MwOperatorNameRetriever
from datetime import datetime

from reportsdata.tipreport import TipReportPresenter, TipReportGenerator
from reportsdata.tipreport.dto import TipReportFilterDto


class ReportGenerator(object):
    @staticmethod
    def generate(mb_context, pos_id, start_date, end_date, operator_id=None):
        start_date = datetime.strptime(start_date, "%Y%m%d")
        end_date = datetime.strptime(end_date, "%Y%m%d")

        repository = MwTipRepository(mb_context)
        operator_retriever = MwOperatorNameRetriever(mb_context)

        generator = TipReportGenerator(TipReportFilterDto(start_date, end_date, operator_id),
                                       operator_retriever,
                                       RealClock(),
                                       repository)

        return generator, TipReportPresenter()



from logging import getLogger

from nperipherals_core.model.repository import NotificationRepository

logger = getLogger(__name__)


class LoggerNotificationRepository(NotificationRepository):
    def notify(self, name, data):
        # type: (str, str) -> None

        logger.info("Printing data [{}] from device [{}]".format(data, name))



from abc import abstractmethod
from logging import getLogger
from typing import Dict, Optional

from messagehandler import MessageHandler
from messagehandler import MessageHandlerBuilder
from messagebus import Message, Event

logger = getLogger(__name__)


class CommonMessageHandlerBuilder(MessageHandlerBuilder):
    def __init__(self, device_configurations):
        # type: (Dict[unicode, unicode]) -> None

        self.device_configurations = device_configurations

    def build_singletons(self):
        # type: () -> None

        return

    @abstractmethod
    def build_message_handler(self, message=None, event=None):
        # type: (Optional[Message], Optional[Event]) -> MessageHandler

        raise NotImplementedError()

    def destroy_message_handler(self, message_handler):
        # type: (MessageHandler) -> None

        return

    def destroy_singletons(self):
        # type: () -> None

        return



from typing import Dict

from simplehttprouter import HttpRouter, Route
from messageprocessor import ApiRouter
from messageprocessor import MessageProcessor


class DefaultApiRouter(ApiRouter):
    def __init__(self, router, processors):
        # type: (HttpRouter, Dict[Route, MessageProcessor]) -> None
        self.router = router
        self.processors = processors

    def get_message_processor(self, api_request):
        route = self.router.get_route(Route(api_request.method, api_request.api_path))
        if route not in self.processors:
            return None
        return self.processors[route]



import logging
from datetime import datetime

from bustoken import TK_SITEF_CONNECTIVITY_TEST, TK_SITEF_AVAILABLE
from msgbus import TK_SYS_ACK, TK_SYS_NAK
from sysactions import action, show_messagebox, get_used_service, get_model

from .. import mb_context

logger = logging.getLogger("PosActions")


@action
def check_tef_connectivity(pos_id, show_dialog="false"):
    try:
        show_dialog = show_dialog.lower() == "true"
        component_name = "Sitef%02d" % int(pos_id)
        data = "{};{};{}".format(pos_id, datetime.now().strftime("%Y%m%d"), datetime.now().strftime('%H:%M:%S'))
        ret = mb_context.MB_EasySendMessage(component_name, TK_SITEF_CONNECTIVITY_TEST, data=data)

        if show_dialog:
            if ret.token == TK_SYS_NAK:
                show_messagebox(pos_id, "$SITEF_IS_NOT_AVAILABLE", "$ERROR")
            else:
                show_messagebox(pos_id, "$SITEF_OK")

        return str(ret.token == TK_SYS_ACK).lower()
    except Exception as ex:
        logger.warning("Error verifying if Electronic Payments is available. Exception: {}".format(ex.message))

        if show_dialog:
            show_messagebox(pos_id, "$SITEF_SERVER_ERROR|{}".format(str(pos_id)), "$ERROR")

        return "false"


@action
def is_tef_available(pos_id):
    model = get_model(pos_id)
    eft_name = get_used_service(model, "eft")

    if eft_name is not None:
        try:
            ret = mb_context.MB_EasySendMessage(eft_name, token=TK_SITEF_AVAILABLE)
            if ret.token == TK_SYS_ACK:
                return ret.data
            return "{}"
        except Exception as _: # noqa
            return "{}"

    return "{}"



import datetime  # noqa


class CashReportFiltersDto(object):
    def __init__(self, report_type, pos_id, store_id, operator_id, initial_date, end_date, session_id, caller_pos_id):
        # type: (int, int, int, datetime, datetime, unicode, int) -> None
        self.report_type = report_type

        self.pos_id = pos_id
        self.store_id = store_id
        self.operator_id = operator_id
        self.initial_date = initial_date
        self.end_date = end_date
        self.session_id = session_id
        self.caller_pos_id = caller_pos_id



from abc import abstractmethod
from typing import Optional
from messageprocessor import MessageProcessor
from uuid import uuid4


class UuidMessageProcessor(MessageProcessor):

    def __init__(self, event_name=None):
        # type: (Optional[str]) -> None
        if event_name is not None:
            self.event_name = event_name
        else:
            self.event_name = type(self).__name__

    def get_processor_name(self):
        # type: () -> str
        if self.event_name is not None:
            return self.event_name

        return self.__class__.__name__

    def generate_unique_id(self):
        # type: () -> str
        return str(uuid4())

    @abstractmethod
    def parse_data(self, data):
        pass

    @abstractmethod
    def call_business(self, obj):
        pass

    @abstractmethod
    def format_response(self, message_bus, message, event, input_obj, result):
        pass

    @abstractmethod
    def format_exception(self, message_bus, message, event, input_obj, exception):
        pass

    @abstractmethod
    def format_parse_exception(self, message_bus, message, event, data, exception):
        pass



from typing import List

from ._DeliveryHeaderDto import DeliveryHeaderDto
from ._DeliveryBodyDto import DeliveryBodyDto


class DeliveryGeneratorDto(object):
    def __init__(self, header, body, products, cfh_products):
        # type: (DeliveryHeaderDto, DeliveryBodyDto, object, List) -> ()
        self.header = header
        self.body = body
        self.products = products
        self.cfh_products = cfh_products



class SaleLine(object):
    def __init__(self, line_number, item_id, level, part_code):
        self.line_number = line_number
        self.item_id = item_id
        self.level = level
        self.part_code = part_code



import json


from ._Report import Report
from ._Part import Part


class ReportParser(object):
    def parse(self, report):
        try:
            report_dict = json.loads(report, "utf-8")
        except:
            return None

        if "key" not in report_dict or report_dict["key"] != Report.key:
            return None

        if "parts" not in report_dict or not isinstance(report_dict["parts"], list):
            return None

        parts = []
        for part_dict in report_dict["parts"]:
            parts.append(Part(part_dict["text"]))

        return parts



from utils import center, double_line_separator

DATE_TIME_FMT = "%d/%m/%Y %H:%M:%S"


def report_default_header(title, pos_id, operator_id, period, current_datetime, store_id=None):
    title = center(title)
    separator = double_line_separator()

    header = separator
    header += title
    if store_id:
        header += "Loja..........: {}".format(store_id.upper()) + "\n"
    header += "Data/hora.....: {}".format(current_datetime) + "\n"

    if period is not None:
        fiscal_date = "%02d/%02d/%04d" % (int(period[6:8]), int(period[4:6]), int(period[:4]))
        header += "Data fiscal...: {}".format(fiscal_date) + "\n"

    if operator_id is not None and pos_id is not None:
        header += "ID Operador ..: {} (POS # {})".format(operator_id, pos_id) + "\n"

    header += separator

    return header



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

# W = Wide bar
# w = wide space
# N = Narrow bar
# n = narrow space

START = 'NnNn'
STOP = 'WnN'
CODES = ('NNWWN', 'WNNNW', 'NWNNW', 'WWNNN', 'NNWNW', 'WNWNN', 'NWWNN', 'NNNWW', 'WNNWN', 'NWNWN')



import time
from logging import Logger
from threading import Thread, Lock

from _ProductionView import ProductionView
from messagebus import MessageBus, TokenCreator, TokenPriority, Message, DataType, DefaultToken
from production.model import ProductionOrder
from production.view._OrderXml import OrderXml
from datetime import datetime

from typing import Dict, List

KDS_GROUP = "B"

TK_KDS_UPDATE_VIEW = TokenCreator.create_token(TokenPriority.high, KDS_GROUP, "2")
TK_KDS_REFRESH_END = TokenCreator.create_token(TokenPriority.high, KDS_GROUP, "4")
TK_KDS_RELOAD = TokenCreator.create_token(TokenPriority.high, KDS_GROUP, "14")


class KdsView(ProductionView):
    def __init__(self, name, message_bus, view_name, kds_manager, logger):
        # type: (str, MessageBus, str, str, Logger) -> None
        super(KdsView, self).__init__(name, logger)
        self.message_bus = message_bus
        self.view_name = view_name

        self.order_xml = OrderXml()
        self.kds_manager = kds_manager

        self.debounce_time = 0.1
        self.orders_timestamp = {}  # type: Dict[int, str]
        self.orders_to_send_lock = Lock()

    def _enqueue_order(self, order):
        # type (ProductionOrder) -> None

        order_id = order.order_id
        now = datetime.now().isoformat()

        with self.orders_to_send_lock:
            self.orders_timestamp[order_id] = now
        self.order_handler(order, now)

    def handle_order(self, order):
        # type: (ProductionOrder) -> Thread

        t = Thread(target=self._enqueue_order, args=[order])
        t.daemon = True
        t.start()
        return t

    def get_view_tags(self):
        # type: () -> List[str]

        return []

    def send_async_message(self, service, message, error_message):
        try:
            reply = self.message_bus.send_message(service, message)
            if reply and reply.token != DefaultToken.TK_SYS_ACK:
                self.debug(error_message, self.name, reply.token, reply.data)
        except Exception as _: # noqa
            self.debug(error_message, self.name, "", "")

    def order_handler(self, order, timestamp):
        # type: (ProductionOrder, str) -> None
        time.sleep(self.debounce_time)
        order_id = order.order_id
        with self.orders_to_send_lock:
            if self.orders_timestamp[order_id] != timestamp:
                return

        xml = self.order_xml.to_xml(order)
        params = "{}\0{}".format(self.view_name, xml)
        message = Message(TK_KDS_UPDATE_VIEW, DataType.param, params, 5000000)
        error_message = '[prod][{{}}] Error sending ORDER {} to KDS VIEW: {}'.format(order.order_id, self.view_name)

        self.send_kds_controller_message(error_message, message)

    def reload_handler(self):
        message = Message(TK_KDS_RELOAD, DataType.param, str(self.view_name), 5000000)
        error_message = '[prod][{{}}] Error sending RELOAD view to KDS VIEW: {}'.format(self.view_name)

        self.send_kds_controller_message(error_message, message)

    def refresh_end(self):
        message = Message(TK_KDS_REFRESH_END, DataType.string, str(self.view_name), 5000000)
        error_message = '[prod][{{}}] Error sending REFRESH view to KDS VIEW: {}'.format(self.view_name)

        self.send_kds_controller_message(error_message, message)

    def send_kds_controller_message(self, error_message, message):
        t = Thread(target=self.send_async_message, args=[self.kds_manager, message, error_message])
        t.daemon = True
        t.start()



from _EscPosFormatter import EscPosFormatter
from escposformatter import CommandFactory
from escposformatter.processor import AlignCommandProcessor, BoldCommandProcessor, RepeatCommandProcessor,\
    FontCommandProcessor, NewLineCommandProcessor
from report.command import AlignCommand, BoldCommand, RepeatCommand, FontCommand, NewLineCommand


class DefaultEscPosFormatter(EscPosFormatter):
    def __init__(self, encoding="cp860"):
        commands = {
            FontCommand: FontCommandProcessor(),
            AlignCommand: AlignCommandProcessor(),
            BoldCommand: BoldCommandProcessor(),
            RepeatCommand: RepeatCommandProcessor(),
            NewLineCommand: NewLineCommandProcessor()
        }
        super(DefaultEscPosFormatter, self).__init__(encoding,
                                                     DefaultEscPosFormatter.DefaultCommandFactory(commands))

    class DefaultCommandFactory(CommandFactory):
        def __init__(self, commands):
            self.commands = commands

        def get_processor(self, command):
            return self.commands[type(command)]



# Needed on case-insensitive filesystems
from __future__ import absolute_import

# Try to import PIL in either of the two ways it can be installed.
try:
    from PIL import Image, ImageDraw
except ImportError:  # pragma: no cover
    import Image
    import ImageDraw

import qrcode.image.base


class PilImage(qrcode.image.base.BaseImage):
    """
    PIL image builder, default format is PNG.
    """
    kind = "PNG"

    def new_image(self, **kwargs):
        back_color = kwargs.get("back_color", "white")
        fill_color = kwargs.get("fill_color", "black")

        if fill_color.lower() != "black" or back_color.lower() != "white":
            if back_color.lower() == "transparent":
                mode = "RGBA"
                back_color = None
            else:
                mode = "RGB"
        else:
            mode = "1"
            # L mode (1 mode) color = (r*299 + g*587 + b*114)//1000
            if fill_color.lower() == "black": fill_color = 0
            if back_color.lower() == "white": back_color = 255

        img = Image.new(mode, (self.pixel_size, self.pixel_size), back_color)
        self.fill_color = fill_color
        self._idr = ImageDraw.Draw(img)
        return img

    def drawrect(self, row, col):
        box = self.pixel_box(row, col)
        self._idr.rectangle(box, fill=self.fill_color)

    def save(self, stream, format=None, **kwargs):
        if format is None:
            format = kwargs.get("kind", self.kind)
        if "kind" in kwargs:
            del kwargs["kind"]
        self._img.save(stream, format=format, **kwargs)

    def __getattr__(self, name):
        return getattr(self._img, name)



from ._CurrencyConversionRepository import CurrencyConversionRepository
from ._I18nRepository import I18nRepository
from ._AccountRepository import AccountRepository
from ._ServicePeriodRepository import ServicePeriodRepository
from ._NavigationRepository import NavigationRepository
from ._OrderRepository import OrderRepository
from ._DrawerRepository import DrawerRepository

__all__ = [
    "CurrencyConversionRepository",
    "I18nRepository",
    "AccountRepository",
    "ServicePeriodRepository",
    "NavigationRepository",
    "OrderRepository",
    "DrawerRepository"
]



# -*- coding: utf-8 -*-
import sysactions
from sysactions import check_operator_logged, StopAction

from msgbus import TK_POS_USERPAUSE, FM_PARAM, TK_SYS_ACK
from .. import logger


@sysactions.action
def pause_user(pos_id):
    try:
        check_operator_logged(pos_id)

        from table_actions import deselect_table
        deselect_table(pos_id)

        msg = sysactions.send_message(
            "POS{}".format(pos_id),
            TK_POS_USERPAUSE,
            FM_PARAM,
            pos_id,
            timeout=60 * 1000000
        )
        if msg.token != TK_SYS_ACK:
            error_code = msg.data.split('\0')[0]
            if error_code == "5003":  # POS BLOCKED BY TIME
                return True

            logger.error("Invalid response from PosCtrl while Pausing user: {} - {}".format(error_code, msg.data))
            sysactions.show_messagebox(pos_id, "$INVALID_USERNAME_OR_PASSWORD", "$INFORMATION")
            return False
    except StopAction:
        return False
    except (Exception, BaseException):
        logger.exception("Exception while pausing user on POS: {}".format(pos_id))
        sysactions.show_messagebox(pos_id, "$INVALID_USERNAME_OR_PASSWORD", "$INFORMATION")
        return False
    return True



# encoding: utf-8
from old_helper import BaseRepository
from ..model import Account


class AccountRepository(BaseRepository):

    def __init__(self, mbcontext):
        super(AccountRepository, self).__init__(mbcontext)

    def get_transfers_by_period(self, period, transfer_type):
        def inner_func(conn):
            query = self._AccountQuery.format(period, transfer_type)
            transfers = []
            for x in conn.select(query):
                transfers.append(Account(float(x.get_entry("Amount")),
                                         x.get_entry("Timestamp"),
                                         x.get_entry("GLAccount")
                                         )
                                 )
            return transfers

        return self.execute_with_connection(inner_func)

    _AccountQuery = """
    SELECT *
    FROM Transfer 
    WHERE period='{}' AND type = '{}'
    """



from pos_model import OrderParser
from xml.etree import cElementTree as eTree

from _GeneralTaxCalculator import GeneralTaxCalculator
from _OrderFormatter import OrderFormatter


class TaxCalculatorService(object):
    def __init__(self, order_parser, general_tax_calculator, order_formatter):
        # type: (OrderParser, GeneralTaxCalculator, OrderFormatter) -> None
        self.order_parser = order_parser
        self.general_tax_calculator = general_tax_calculator
        self.order_formatter = order_formatter

    def calculate_taxes(self, order_str):
        # type: (str) -> unicode
        xml = eTree.XML(order_str)
        root = xml if xml.tag == "Order" else None
        order_xml = root or xml.find(".//Order")
        if not order_xml:
            raise Exception("Invalid order XML")

        order = self.order_parser.parse_order(order_xml)
        for sale_item in order.sale_items:
            tax_items = self.general_tax_calculator.calculate_all_taxes(sale_item)
            sale_item.taxes = tax_items

        formatted_order = self.order_formatter.format_order(order)

        return formatted_order

    def insert_tx_idx(self, order_str):
        # type: (str) -> unicode
        order_xml = eTree.XML(order_str).find("Order")
        for sale_line in order_xml.findall("SaleLine"):
            if float(sale_line.get("qty") or 1) == 0:
                continue

            sale_item = self.order_parser.create_sale_item(sale_line)
            tax_items = self.general_tax_calculator.calculate_all_taxes(sale_item)
            sale_line_price = sale_item.unit_price if sale_item.unit_price > 0 else sale_item.added_unit_price
            if tax_items and float(sale_line_price) > 0:
                sale_item.taxes = tax_items
                tax_item = sale_item.get_tax_item("ICMS")
                if tax_item is None:
                    continue

                sale_line.set("taxIndex", tax_item.tax_index)
        return eTree.tostring(order_xml)



from _OrderRepository import OrderRepository
from _ProductRepository import ProductRepository
from _SaleChallengeRepository import SaleChallengeRepository

__all__ = ["OrderRepository", "ProductRepository", "SaleChallengeRepository"]



from common import state_match, check_and_remove
from . import StatisticBox


class ReadingPercentageStatisticBox(StatisticBox):
    def __init__(self, name, sons, statistic_updater, logger, get_config):
        super(ReadingPercentageStatisticBox, self).__init__(name, sons, statistic_updater, logger, get_config)
        self.orders = []
        self.orders_matched = []
        self.state_value = get_config("State").get("Value")
        self.state_type = get_config("State").get("Type")

    def update_statistics(self, order):
        order_id = order.order_id
        if order_id not in self.orders:
            self.orders.append(order_id)

        check_and_remove(self.orders, order, self.state_type, self.state_value, False)
        check_and_remove(self.orders_matched, order, self.state_type, self.state_value)

        if order_id not in self.orders_matched and state_match(order, self.state_type, self.state_value):
            self.orders_matched.append(order_id)

        return "{:.2f}".format((100 * (float(len(self.orders_matched)) / (len(self.orders) or 1))))



# -*- coding: utf-8 -*-

import json
import os

import cfgtools
import sysactions
from actions import mb_context
from persistence import Driver


@sysactions.action
def get_kiosk_configurations(pos_id):
    config = cfgtools.read(os.environ["LOADERCFG"])
    screen_timeout = config.find_value("KioskConfig.ScreenTimeout")
    cancel_timeout = config.find_value("KioskConfig.CancelTimeoutWindow")
    top_banner_timeout = config.find_value("KioskConfig.TopBannerTimeout")
    kiosk_context = config.find_value("KioskConfig.KioskContext")
    show_sale_type = (config.find_value("KioskConfig.ShowSaleType") or "false").lower() == "true"
    supported_langs = config.find_values("KioskConfig.SupportedLangs")

    # TODO: get kiosk configuration correctly
    params = {
        'descr_id': 1
    }
    conn = None
    try:
        conn = Driver().open(mb_context, pos_id, service_name="Persistence")
        cursor = conn.pselect("getDescriptions", **params)
        descriptions = {row.get_entry(0): row.get_entry(1) for row in cursor}
    finally:
        if conn:
            conn.close()

    config = {
        'supportedLangs': supported_langs,
        'screenTimeout': int(screen_timeout),
        'cancelTimeoutWindow': int(cancel_timeout),
        'topBannerTimeout': int(top_banner_timeout),
        'showSaleType': show_sale_type,
        'baseContext': kiosk_context,
        'descriptions': descriptions
    }
    return json.dumps(config)



class ConfigurationReaderException(Exception):
    pass


class InvalidMessageContentException(Exception):
    pass


class OrderNotFoundException(Exception):
    def __init__(self):
        super(Exception, self).__init__("NDISCOUNT_ORDER_NOT_FOUND")


class BenefitNotFound(Exception):
    pass


class TimeoutException(Exception):
    def __init__(self):
        super(Exception, self).__init__("NDISCOUNT_TIMEOUT")


class ProductNotFound(Exception):
    def __init__(self):
        super(Exception, self).__init__("NDISCOUNT_PRODUCT_NOT_FOUND")


class InvalidTenderIdException(Exception):
    def __init__(self):
        super(Exception, self).__init__("NDISCOUNT_INVALID_TENDER_ID")


class InvalidAmountException(Exception):
    def __init__(self):
        super(Exception, self).__init__("NDISCOUNT_INVALID_AMOUNT")


class InvalidPercentageException(Exception):
    def __init__(self):
        super(Exception, self).__init__("NDISCOUNT_INVALID_PERCENTAGE")


class NotEffectiveDiscountException(Exception):
    def __init__(self):
        super(Exception, self).__init__("NDISCOUNT_NOT_EFFECTIVE_DISCOUNT")


class InvalidOperationException(Exception):
    def __init__(self):
        super(Exception, self).__init__("NDISCOUNT_INVALID_OPERATION")


class InvalidQuantityException(Exception):
    def __init__(self):
        super(Exception, self).__init__()



from json import loads, dumps

from typing import List, Dict

from . import Benefit


class OrderBenefitInfo(object):
    def __init__(self, benefit, used_discount_ids, added_sale_lines, burnt_items, promotion_tender,
                 discount_item_values, order_state=None, benefit_applied=False, benefit_key=None, times_to_apply=None):
        # type: (Dict, list[int], list[int], Dict, List, Dict, str, bool, str, int) -> None

        self.benefit = Benefit(benefit)
        self.used_discount_ids = used_discount_ids
        self.added_sale_lines = added_sale_lines
        self.burnt_items = burnt_items
        self.order_state = order_state
        self.promotion_tender = promotion_tender
        self.discount_item_values = discount_item_values
        self.benefit_applied = benefit_applied
        self.benefit_key = benefit_key
        self.times_to_apply = times_to_apply

    @staticmethod
    def from_json(json):
        benefit = json.get("benefit", "{}")
        used_discount_ids = json.get("used_discount_ids", [])
        added_sale_lines = json.get("added_sale_lines", [])
        burnt_items = json.get("burnt_items", {})
        order_state = json.get("order_state")
        item_values = json.get("discount_item_values", {})
        tenders = json.get("promotion_tender", [])
        benefit_applied = json.get("benefit_applied", [])
        benefit_key = json.get("benefit_key", None)
        times_to_apply = json.get("times_to_apply", None)

        return OrderBenefitInfo(benefit, used_discount_ids, added_sale_lines, burnt_items, tenders, item_values,
                                order_state, benefit_applied, benefit_key, times_to_apply)

    def to_json(self):
        # type: () -> str

        return dumps({
            "benefit": loads(self.benefit.to_json()),
            "used_discount_ids": list(self.used_discount_ids),
            "added_sale_lines": list(self.added_sale_lines),
            "burnt_items": self.burnt_items if self.benefit.conditions.burn_items else dict(),
            "order_state": self.order_state,
            "promotion_tender": self.promotion_tender,
            "discount_item_values": self.discount_item_values,
            "times_to_apply": self.times_to_apply
        })



from commons.dto import ReportColumnDefinition, AlignTypes
from commons.report import Formatter
from commons.util import TableReport, DefaultHeaderFormatter
from reports_app.cashreport.dto import CashReportDto  # noqa


class CashReportFormatter(Formatter):
    def __init__(self):
        super(CashReportFormatter, self).__init__()
        self.translated_messages = {}

    def format_report(self, cash_report_dto, translator):
        # type: (CashReportDto) -> None
        self.translated_messages = translator.translate_labels(["$CASH_REPORT_BY_REAL_DATE_REPORT_TITLE",
                                                                "$CASH_REPORT_BY_BUSINESS_PERIOD_REPORT_TITLE",
                                                                "$CASH_REPORT_BY_XML_REPORT_TITLE",
                                                                "$CASH_REPORT_BY_SESSION_ID_REPORT_TITLE",
                                                                "$TYPE",
                                                                "$QTY",
                                                                "$VALUE",
                                                                "$INITIAL_BALANCE",
                                                                "$SALES",
                                                                "$VOIDED_ORDERS",
                                                                "$DISCOUNTS",
                                                                "$DONATIONS",
                                                                "$LIQUID_SALES",
                                                                "$TENDER_TYPES",
                                                                "$CASH_OUT",
                                                                "$CASH_IN",
                                                                "$VALUE_IN_DRAWER"
                                                                ],
                                                               cash_report_dto.pos_id)

        report = self._append_header(cash_report_dto, translator)
        report += self._append_body(cash_report_dto)
        report_bytes = report.decode("utf-8").encode("utf-8")
        return report_bytes

    def _append_body(self, cash_report_dto):
        report_body_lines = []

        self._append_report_body_title(report_body_lines)
        self._append_body_items(cash_report_dto, report_body_lines)
        self._append_line_separator(report_body_lines)
        return self._format_body(report_body_lines)

    @staticmethod
    def _append_line_separator(report_body_lines):
        report_body_lines.append([ReportColumnDefinition(
            fill_with_char='='
        )])

    @staticmethod
    def _format_body(report_body_lines):
        table_formatter = TableReport([
            ReportColumnDefinition(text='', width=17, fill_with_char='.', align=AlignTypes.LEFT),
            ReportColumnDefinition(text=': [', width=3),
            ReportColumnDefinition(text='', width=4, fill_with_char='', before_text='', after_text='',
                                   align=AlignTypes.RIGHT),
            ReportColumnDefinition(text='] R$', width=4),
            ReportColumnDefinition(text='', width=10, fill_with_char='', before_text='', after_text='',
                                   align=AlignTypes.RIGHT)
        ])
        return table_formatter.format(report_body_lines)

    def _append_body_items(self, cash_report_dto, report_body_lines):
        for line in cash_report_dto.report_body_dto.cash_report_body_lines_dto:
            self._append_transfers_info(line, report_body_lines)

    def _append_transfers_info(self, report_item, report_body_lines):
        line_type = self.translated_messages["" if report_item.name is None else report_item.name].upper()
        separator_1 = "" if report_item.quantity is None else ": ["

        if report_item.quantity is None:
            quantity = ""
            separator_1 = ":  "
        else:
            quantity = str(report_item.quantity)

        if report_item.value is None and report_item.quantity is None:
            separator_2 = ""
        elif report_item.quantity is None:
            separator_2 = "  R$"
        elif report_item.value is None:
            separator_2 = "]   "
        else:
            separator_2 = "] R$"

        if report_item.value is None:
            value = ""
        else:
            value = str(report_item.value)

        report_body_lines.append([line_type,
                                  separator_1,
                                  quantity,
                                  separator_2,
                                  value])

    def _append_header(self, cash_report_dto, translator):
        default_header_formatter = DefaultHeaderFormatter(translator)
        if cash_report_dto.report_header_dto.report_type == 'report_by_real_date':
            title = self.translated_messages["$CASH_REPORT_BY_REAL_DATE_REPORT_TITLE"]
        if cash_report_dto.report_header_dto.report_type == 'report_by_business_period':
            title = self.translated_messages["$CASH_REPORT_BY_BUSINESS_PERIOD_REPORT_TITLE"]
        if cash_report_dto.report_header_dto.report_type == 'report_by_session_id':
            title = self.translated_messages["$CASH_REPORT_BY_SESSION_ID_REPORT_TITLE"]
        if cash_report_dto.report_header_dto.report_type == 'report_by_xml':
            title = self.translated_messages["$CASH_REPORT_BY_XML_REPORT_TITLE"]

        return default_header_formatter.format_header(title, cash_report_dto.report_header_dto, cash_report_dto.pos_id)

    def _append_report_body_title(self, report_body_lines):
        report_body_lines.append([
            ReportColumnDefinition(text=self.translated_messages["$TYPE"], width=17),
            ReportColumnDefinition(text='', width=3),
            ReportColumnDefinition(text=self.translated_messages["$QTY"], width=4, align=AlignTypes.RIGHT),
            ReportColumnDefinition(text='', width=4),
            ReportColumnDefinition(text=self.translated_messages["$VALUE"] + '(R$)', width=10, align=AlignTypes.RIGHT)
        ])



import persistence
from systools import sys_log_exception

from .. import mb_context


def get_payments_amount_by_type(pos_id, session_id, payments_amount):
    conn = None

    try:
        conn = persistence.Driver().open(mb_context, str(pos_id))
        cursor = conn.select("""
            SELECT ot.TenderId, sum(ot.TenderAmount - COALESCE(ot.ChangeAmount, 0)) as Amount
            FROM OrderTender ot
            INNER JOIN Orders o ON o.OrderId = ot.OrderId
            WHERE Sessionid = '{0}' AND StateId = 5 AND ot.OrderId NOT IN (
                SELECT DISTINCT ocp.OrderId 
                FROM OrderCustomProperties ocp
                INNER JOIN Orders o ON ocp.OrderId = o.OrderId
                WHERE ocp.Key = 'DELIVERY_ORDER_MANUAL_CONFIRMED' AND o.SessionId = '{0}'
            )
            GROUP BY ot.TenderId""".format(session_id))

        if cursor.rows() > 0:
            for row in cursor:
                if row.get_entry("TenderId") not in payments_amount:
                    payments_amount[row.get_entry("TenderId")] = float(row.get_entry("Amount") or 0.0)
                else:
                    current_amount = payments_amount[row.get_entry("TenderId")]
                    payments_amount[row.get_entry("TenderId")] = current_amount + float(row.get_entry("Amount") or 0.0)

    except Exception as _:
        sys_log_exception("Error getting ")
    finally:
        if conn:
            conn.close()

    return payments_amount



from ._MessageProcessor import MessageProcessor
from ._DefaultMessageProcessorExecutor import DefaultMessageProcessorExecutor
from ._MessageProcessorMessageHandler import MessageProcessorMessageHandler
from ._MessageProcessorCallback import MessageProcessorCallback
from ._LoggingProcessorCallback import LoggingProcessorCallback
from ._ApiRouter import ApiRouter
from ._ApiRequest import ApiRequest
from ._MessageProcessorExecutorFactory import MessageProcessorExecutorFactory
from ._MessageProcessorExecutor import MessageProcessorExecutor
from ._DefaultMessageProcessorExecutor import DefaultMessageProcessorExecutor
from ._DefaultMessageProcessorExecutorFactory import DefaultMessageProcessorExecutorFactory
from ._TimeProcessorCallback import TimeProcessorCallback



from production.box import ItemPropertiesFilterBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class ItemPropertiesFilterBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(ItemPropertiesFilterBoxBuilder, self).__init__(loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> ItemPropertiesFilterBox
        return ItemPropertiesFilterBox(config.name,
                                       config.sons,
                                       self.get_extra(config, "AllowedProperties", None),
                                       self.get_extra(config, "ForbiddenProperties", None),
                                       self.get_extra(config, "AllowComboProperties", 'False'),
                                       self.get_logger(config))



from logging import getLogger

from messagebus import DataType, DefaultToken, Message
from messageprocessorutil import UuidMessageProcessor
from nperipherals_core.interactor.print_interactor import PrinterStatusInteractor
from nperipherals_core.model.device_status import DeviceStatus

logger = getLogger(__name__)


class PrinterStatusProcessor(UuidMessageProcessor):
    def __init__(self, interactor):
        # type: (PrinterStatusInteractor) -> None

        super(PrinterStatusProcessor, self).__init__()
        self.interactor = interactor

    def parse_data(self, data, event_type):
        # we don't expect data
        pass

    def call_business(self, obj):
        return self.interactor.get_device_status()

    def format_response(self, message_bus, message, event, input_obj, result):

        if result == DeviceStatus.Ok:
            message_bus.reply_message(message, Message(DefaultToken.TK_SYS_ACK, DataType.string, "0"))
        else:
            message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.string, "1"))

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.string, str(exception)))

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.string, str(exception)))



import json

import sysactions

import get_products
from .. import logger


@sysactions.action
def open_rupture(pos_id):
    try:
        enabled_ingredients = get_products.get_enabled_products()
        disabled_ingredients = get_products.get_disabled_products()

    except BaseException as e:
        logger.exception('[rupture_items] Exception {}'.format(e))
        sysactions.show_messagebox(pos_id, "$ERROR_GETTING_DATA", "$INFORMATION", buttons="Sair")
        return

    dialog_contents = {
        "enabledIngredients": enabled_ingredients,
        "disabledIngredients": disabled_ingredients
    }

    info_text = json.dumps(dialog_contents)
    sysactions.show_any_dialog(pos_id, "rupture", "", "", info_text, "", "", 6000000, "", "", "", "", "", False)



from messagebus import AckMessage, NakMessage
from messageprocessorutil import UuidMessageProcessor
from production.manager import ProductionManager
from production.model.request import GetOrderXmlRequest
from production.view import OrderXml


class GetOrderXmlProcessor(UuidMessageProcessor):
    def __init__(self, production_manager):
        # type: (ProductionManager) -> None
        super(GetOrderXmlProcessor, self).__init__()
        self.production_manager = production_manager
        self.order_xml = OrderXml()

    def parse_data(self, data):
        return GetOrderXmlRequest(int(data))

    def call_business(self, request):
        production_order = self.production_manager.get_production_order(request.order_id)
        if production_order is None:
            return None
        return self.order_xml.to_xml(production_order)

    def format_response(self, message_bus, message, event, input_obj, order_xml):
        if order_xml is None:
            message_bus.reply_message(message, NakMessage("order not found"))
            return

        message_bus.reply_message(message, AckMessage(order_xml))

    def format_exception(self, message_bus, message, event, input_obj, exception):
        exception_message = "Error retrieving order xml: {}".format(exception)
        message_bus.reply_message(message, NakMessage(exception_message))

    def format_parse_exception(self, message_bus, message, event, data, exception):
        exception_message = "Error parsing received data: {}; Received data: {}".format(exception, data)
        message_bus.reply_message(message, NakMessage(exception_message))



"""
A dict subclass for Python 2 that behaves like Python 3's dict

Example use:

>>> from builtins import dict
>>> d1 = dict()    # instead of {} for an empty dict
>>> d2 = dict(key1='value1', key2='value2')

The keys, values and items methods now return iterators on Python 2.x
(with set-like behaviour on Python 2.7).

>>> for d in (d1, d2):
...     assert not isinstance(d.keys(), list)
...     assert not isinstance(d.values(), list)
...     assert not isinstance(d.items(), list)
"""

import sys

from future.utils import with_metaclass
from future.types.newobject import newobject


_builtin_dict = dict
ver = sys.version_info[:2]


class BaseNewDict(type):
    def __instancecheck__(cls, instance):
        if cls == newdict:
            return isinstance(instance, _builtin_dict)
        else:
            return issubclass(instance.__class__, cls)


class newdict(with_metaclass(BaseNewDict, _builtin_dict)):
    """
    A backport of the Python 3 dict object to Py2
    """
    def items(self):
        """
        On Python 2.7+:
            D.items() -> a set-like object providing a view on D's items
        On Python 2.6:
            D.items() -> an iterator over D's items
        """
        if ver == (2, 7):
            return self.viewitems()
        elif ver == (2, 6):
            return self.iteritems()
        elif ver >= (3, 0):
            return self.items()

    def keys(self):
        """
        On Python 2.7+:
            D.keys() -> a set-like object providing a view on D's keys
        On Python 2.6:
            D.keys() -> an iterator over D's keys
        """
        if ver == (2, 7):
            return self.viewkeys()
        elif ver == (2, 6):
            return self.iterkeys()
        elif ver >= (3, 0):
            return self.keys()

    def values(self):
        """
        On Python 2.7+:
            D.values() -> a set-like object providing a view on D's values
        On Python 2.6:
            D.values() -> an iterator over D's values
        """
        if ver == (2, 7):
            return self.viewvalues()
        elif ver == (2, 6):
            return self.itervalues()
        elif ver >= (3, 0):
            return self.values()

    def __new__(cls, *args, **kwargs):
        """
        dict() -> new empty dictionary
        dict(mapping) -> new dictionary initialized from a mapping object's
            (key, value) pairs
        dict(iterable) -> new dictionary initialized as if via:
            d = {}
            for k, v in iterable:
                d[k] = v
        dict(**kwargs) -> new dictionary initialized with the name=value pairs
            in the keyword argument list.  For example:  dict(one=1, two=2)
        """

        if len(args) == 0:
            return super(newdict, cls).__new__(cls)
        elif type(args[0]) == newdict:
            value = args[0]
        else:
            value = args[0]
        return super(newdict, cls).__new__(cls, value)
        
    def __native__(self):
        """
        Hook for the future.utils.native() function
        """
        return dict(self)


__all__ = ['newdict']
